@@ -1,5 +1,5 @@
-/* GStreamer
- * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
+/* GStreamer media-info library
+ * Copyright (C) 2003,2004 Thomas Vander Stichele <thomas@apestaart.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -34,6 +34,8 @@ static void	gst_media_info_get_property     (GObject *object, guint prop_id,
 						 GParamSpec *pspec);
 
 
+static gboolean _media_info_inited = FALSE;
+
 /* FIXME: this is a lousy hack that needs to go */
 #define MAX_METADATA_ITERS 5
 
@@ -48,16 +50,17 @@ enum
 
 static guint gst_media_info_signals[LAST_SIGNAL] = { 0 };
 
-/* GError stuff */
 /*
+ * all GError stuff
+ */
+
 enum
 {
   MEDIA_INFO_ERROR_FILE
 };
-*/
+
 /* GError quark stuff */
-/*
-static GQuark
+GQuark
 gst_media_info_error_quark (void)
 {
   static GQuark quark = 0;
@@ -65,7 +68,7 @@ gst_media_info_error_quark (void)
     quark = g_quark_from_static_string ("gst-media-info-error-quark");
   return quark;
 }
-*/
+
 /*
  * GObject type stuff
  */
@@ -77,6 +80,21 @@ enum
 
 static GObjectClass *parent_class = NULL;
 
+GST_DEBUG_CATEGORY (gst_media_info_debug);
+
+/* initialize the media-info library */
+void
+gst_media_info_init (void)
+{
+  if (_media_info_inited) return;
+
+  /* register our debugging category */
+  GST_DEBUG_CATEGORY_INIT (gst_media_info_debug, "GST_MEDIA_INFO", 0,
+                           "GStreamer media-info library");
+  GST_DEBUG ("Initialized media-info library");
+  _media_info_inited = TRUE;
+}
+
 GType
 gst_media_info_get_type (void)
 {
@@ -125,7 +143,7 @@ gst_media_info_class_init (GstMediaInfoClass *klass)
   gst_media_info_signals [MEDIA_INFO_SIGNAL] =
     g_signal_new ("media-info",
 		  G_TYPE_FROM_CLASS (klass),
-		  G_SIGNAL_RUN_FIRST,
+		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GstMediaInfoClass, media_info_signal),
 		  NULL, NULL,
 		  gst_marshal_VOID__VOID,
@@ -135,48 +153,15 @@ gst_media_info_class_init (GstMediaInfoClass *klass)
 static void
 gst_media_info_instance_init (GstMediaInfo *info)
 {
-  GstElement *source;
+  GError **error;
 
   info->priv = g_new0 (GstMediaInfoPriv, 1);
+  error = &info->priv->error;
 
-  info->priv->pipeline = gst_pipeline_new ("media-info");
+  if (!_media_info_inited) { gst_media_info_init (); }
 
-  info->priv->typefind = gst_element_factory_make ("typefind", "typefind");
-  if (!GST_IS_ELEMENT (info->priv->typefind))
-	  /* FIXME */
-    g_error ("Cannot create typefind element !");
-
-  /* ref it so it never goes away on removal out of bins */
-  gst_object_ref (GST_OBJECT (info->priv->typefind));
-
-  /* use gnomevfssrc by default */
-  source = gst_element_factory_make ("gnomevfssrc", "source");
-  if (GST_IS_ELEMENT (source))
-  {
-    info->priv->source = source;
-    info->priv->source_element = g_strdup ("gnomevfssrc");
-    gst_bin_add (GST_BIN (info->priv->pipeline), info->priv->source);
-  }
-  else
-  {
-    info->priv->source = NULL;
-    info->priv->source_element = NULL;
-  }
-  info->priv->location = NULL;
-  info->priv->decoder = NULL;
-  info->priv->type = NULL;
-  info->priv->format = NULL;
-  info->priv->metadata = NULL;
-
-  /* clear result pointers */
-  info->priv->stream = NULL;
-
-  /* set up decoder hash table */
-  info->priv->decoders = g_hash_table_new (g_str_hash, g_str_equal);
-
-  /* attach notify handler */
-  g_signal_connect (G_OBJECT (info->priv->pipeline), "deep_notify",
-		    G_CALLBACK (deep_notify_callback), info->priv);
+  gmip_init (info->priv, error);
+  gmip_reset (info->priv);
 }
 
 /* get/set */
@@ -189,7 +174,7 @@ gst_media_info_get_property (GObject *object, guint prop_id,
   switch (prop_id)
   {
     case PROP_SOURCE:
-      g_value_set_string (value, info->priv->source_element);
+      g_value_set_string (value, info->priv->source_name);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -198,12 +183,24 @@ gst_media_info_get_property (GObject *object, guint prop_id,
 }
 
 GstMediaInfo *
-gst_media_info_new (const gchar *source_element)
+gst_media_info_new (GError **error)
 {
   GstMediaInfo *info = g_object_new (GST_MEDIA_INFO_TYPE, NULL);
-  if (source_element)
-    g_object_set (G_OBJECT (info), "source", source_element);
 
+  if (info->priv->error)
+  {
+    if (error)
+    {
+      *error = info->priv->error;
+      info->priv->error = NULL;
+    }
+    else
+    {
+      g_warning ("Error creating GstMediaInfo object.\n%s",
+                 info->priv->error->message);
+      g_error_free (info->priv->error);
+    }
+  }
   return info;
 }
 
@@ -211,28 +208,9 @@ gst_media_info_new (const gchar *source_element)
  * public methods
  */
 gboolean
-gst_media_info_set_source (GstMediaInfo *info, const char *source)
+gst_media_info_set_source (GstMediaInfo *info, const char *source, GError **error)
 {
-  GstElement *src;
-  src = gst_element_factory_make (source, "new-source");
-  if  (!GST_IS_ELEMENT (src))
-    return FALSE;
-
-  if (info->priv->source)
-  {
-    /* this also unrefs the element */
-    gst_bin_remove (GST_BIN (info->priv->pipeline), info->priv->source);
-    if (info->priv->source_element)
-    {
-      g_free (info->priv->source_element);
-      info->priv->source_element = NULL;
-    }
-  }
-  g_object_set (G_OBJECT (src), "name", "source", NULL);
-  gst_bin_add (GST_BIN (info->priv->pipeline), src);
-  info->priv->source = src;
-  info->priv->source_element = g_strdup (source);
-
+  info->priv->source_name = g_strdup (source);
   return TRUE;
 }
 
@@ -244,11 +222,11 @@ gst_media_info_set_source (GstMediaInfo *info, const char *source)
  */
 void
 gst_media_info_read_with_idler (GstMediaInfo *info, const char *location,
-		                guint16 flags)
+		                guint16 flags, GError **error)
 {
   GstMediaInfoPriv *priv = info->priv;
 
-  gmi_reset (info);		/* reset all structs */
+  gmip_reset (info->priv);		/* reset all structs */
   priv->location = g_strdup (location);
   priv->flags = flags;
 }
@@ -258,30 +236,42 @@ gst_media_info_read_with_idler (GstMediaInfo *info, const char *location,
  * returns: TRUE if it was able to idle, FALSE if there was an error
  */
 gboolean
-gst_media_info_read_idler (GstMediaInfo *info, GstMediaInfoStream **streamp)
+gst_media_info_read_idler (GstMediaInfo *info, GstMediaInfoStream **streamp, GError **error)
 {
-  GstMediaInfoPriv *priv ;
+  GstMediaInfoPriv *priv;
+
   /* if it's NULL then we're sure something went wrong higher up) */
   if (info == NULL) return FALSE;
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   priv = info->priv;
 
   g_assert (streamp != NULL);
+  g_assert (priv);
+
   switch (priv->state)
   {
     case GST_MEDIA_INFO_STATE_NULL:
+      /* make sure we have a source */
+      if (!priv->source_name)
+      {
+        *error = g_error_new (GST_MEDIA_INFO_ERROR, 0,
+                              "No source set on media info.");
+        return FALSE;
+      }
+
       /* need to find type */
-      GMI_DEBUG("idler: NULL, need to find type\n");
-      return gmip_find_type_pre (priv);
+      GST_DEBUG ("idler: NULL, need to find type, priv %p", priv);
+      return gmip_find_type_pre (priv, error);
 
     case GST_MEDIA_INFO_STATE_TYPEFIND:
     {
       gchar *mime;
-      GstElement *decoder;
 
-      GMI_DEBUG("STATE_TYPEFIND\n");
+      GST_LOG ("STATE_TYPEFIND");
       if ((priv->type == NULL) && gst_bin_iterate (GST_BIN (priv->pipeline)))
       {
+	GST_DEBUG ("iterating while in STATE_TYPEFIND");
 	GMI_DEBUG("?");
         return TRUE;
       }
@@ -291,29 +281,28 @@ gst_media_info_read_idler (GstMediaInfo *info, GstMediaInfoStream **streamp)
 	return FALSE;
       }
       /* do the state transition */
-      GMI_DEBUG("doing find_type_post\n");
+      GST_DEBUG ("doing find_type_post");
       gmip_find_type_post (priv);
-      GMI_DEBUG("finding out mime type\n");
-      mime = g_strdup (gst_caps_get_mime (priv->type));
-      GMI_DEBUG("found out mime type: %s\n", mime);
-      decoder = gmi_get_decoder (info, mime);
-      if (decoder == NULL)
+      GST_DEBUG ("finding out mime type");
+      mime = g_strdup (gst_structure_get_name (
+	    gst_caps_get_structure(priv->type, 0)));
+      GST_DEBUG ("found out mime type: %s", mime);
+      if (!gmi_set_mime (info, mime))
       {
         /* FIXME: pop up error */
-        GMI_DEBUG("DEBUG: no decoder found for mime %s\n", mime);
+        GST_DEBUG ("no decoder pipeline found for mime %s", mime);
         return FALSE;
       }
       priv->stream = gmi_stream_new ();
-      GMI_DEBUG("DEBUG: new stream: %p\n", priv->stream);
+      GST_DEBUG ("new stream: %p", priv->stream);
       priv->stream->mime = mime;
       priv->stream->path = priv->location;
-      gmi_set_decoder (info, decoder);
 
       gmip_find_stream_pre (priv);
     }
     case GST_MEDIA_INFO_STATE_STREAM:
     {
-      GMI_DEBUG("STATE_STREAM\n");
+      GST_LOG ("STATE_STREAM");
       if ((priv->format == NULL) && gst_bin_iterate (GST_BIN (priv->pipeline)))
       {
 	GMI_DEBUG("?");
@@ -347,11 +336,11 @@ gst_media_info_read_idler (GstMediaInfo *info, GstMediaInfoStream **streamp)
       if (priv->metadata == NULL)
       {
 	/* this is not a permanent failure */
-        GMI_DEBUG("Couldn't find metadata\n");
+        GST_DEBUG ("Couldn't find metadata");
       }
-      GMI_DEBUG("found metadata of track %ld\n", priv->current_track_num);
+      GST_DEBUG ("found metadata of track %ld", priv->current_track_num);
       if (!gmip_find_track_metadata_post (priv)) return FALSE;
-      GMI_DEBUG("METADATA: going to STREAMINFO\n");
+      GST_DEBUG ("METADATA: going to STREAMINFO\n");
       priv->state = GST_MEDIA_INFO_STATE_STREAMINFO;
       return gmip_find_track_streaminfo_pre (priv);
     }
@@ -366,10 +355,10 @@ gst_media_info_read_idler (GstMediaInfo *info, GstMediaInfoStream **streamp)
       if (priv->streaminfo == NULL)
       {
 	/* this is not a permanent failure */
-        GMI_DEBUG("Couldn't find streaminfo\n");
+        GST_DEBUG ("Couldn't find streaminfo");
       }
       else
-        GMI_DEBUG("found streaminfo of track %ld\n", priv->current_track_num);
+        GST_DEBUG ("found streaminfo of track %ld", priv->current_track_num);
       if (!gmip_find_track_streaminfo_post (priv)) return FALSE;
       priv->state = GST_MEDIA_INFO_STATE_FORMAT;
       return gmip_find_track_format_pre (priv);
@@ -387,7 +376,7 @@ gst_media_info_read_idler (GstMediaInfo *info, GstMediaInfoStream **streamp)
         g_warning ("Couldn't find format\n");
 	return FALSE;
       }
-      GMI_DEBUG("found format of track %ld\n", priv->current_track_num);
+      GST_DEBUG ("found format of track %ld", priv->current_track_num);
       if (!gmip_find_track_format_post (priv)) return FALSE;
       /* save the track info */
       priv->stream->tracks = g_list_append (priv->stream->tracks,
@@ -410,10 +399,10 @@ gst_media_info_read_idler (GstMediaInfo *info, GstMediaInfoStream **streamp)
 	return TRUE;
       }
       priv->state = GST_MEDIA_INFO_STATE_DONE;
+      gmi_clear_decoder (info);
+      GST_DEBUG ("TOTALLY DONE, setting pointer *streamp to %p", *streamp);
       *streamp = priv->stream;
       priv->stream = NULL;
-      GMI_DEBUG("TOTALLY DONE, setting pointer *streamp to %p\n", *streamp);
-      gmi_clear_decoder (info);
       return TRUE;
     }
     case GST_MEDIA_INFO_STATE_DONE:
@@ -428,74 +417,17 @@ gst_media_info_read_idler (GstMediaInfo *info, GstMediaInfoStream **streamp)
  * read all possible info from the file pointed to by location
  * use flags to limit the type of information searched for */
 GstMediaInfoStream *
-gst_media_info_read (GstMediaInfo *info, const char *location, guint16 flags)
+gst_media_info_read (GstMediaInfo *info, const char *location, guint16 flags, GError **error)
 {
-  GstMediaInfoPriv *priv = info->priv;
   GstMediaInfoStream *stream = NULL;
-  GstElement *decoder = NULL;
-  gchar *mime;
-  int i;
 
-  GMI_DEBUG("DEBUG: gst_media_info_read: start\n");
-  gmi_reset (info);		/* reset all structs */
-  priv->location = g_strdup (location);
-  priv->flags = flags;
-
-  if (!gmip_find_type (priv)) return NULL;
-
-  mime = g_strdup (gst_caps_get_mime (priv->type));
-  GMI_DEBUG("mime type: %s\n", mime);
-
-  /* c) figure out decoder */
-  decoder = gmi_get_decoder (info, mime);
-  g_print ("DEBUG: using decoder %s\n", gst_element_get_name (decoder));
-
- /* if it's NULL, then that's a sign we can't decode it */
-  if (decoder == NULL)
-  {
-    g_warning ("Can't find a decoder for type %s\n", mime);
+  gst_media_info_read_with_idler (info, location, flags, error);
+  if (*error) return FALSE;
+  while (gst_media_info_read_idler (info, &stream, error) && stream == NULL)
+    /* keep looping */;
+  if (*error)
     return NULL;
-  }
 
-  /* b) create media info stream object */
-  priv->stream = gmi_stream_new ();
-  priv->stream->mime = mime;
-  priv->stream->path = priv->location;
-
-  /* install this decoder in the pipeline */
-  gmi_set_decoder (info, decoder);
-
-  /* collect total stream properties */
-  /* d) get all stream properties */
-  gmip_find_stream (priv);
-
-  /* e) if we have multiple tracks, loop over them; if not, just get
-   * metadata and return it */
-  GMI_DEBUG("DEBUG: num tracks %ld\n", priv->stream->length_tracks);
-  for (i = 0; i < priv->stream->length_tracks; ++i)
-  {
-    priv->current_track = gmi_track_new ();
-    if (i > 0)
-    {
-      GMI_DEBUG("seeking to track %d\n", i);
-      gmi_seek_to_track (info, i);
-    }
-    if (flags & GST_MEDIA_INFO_METADATA)
-      gmip_find_track_metadata (priv);
-    if (flags & GST_MEDIA_INFO_STREAMINFO)
-      gmip_find_track_streaminfo (priv);
-    if (flags & GST_MEDIA_INFO_FORMAT)
-      gmip_find_track_format (priv);
-    priv->stream->tracks = g_list_append (priv->stream->tracks,
-		                          priv->current_track);
-    priv->current_track = NULL;
-  }
-
-  /* f) clear decoder */
-  gmi_clear_decoder (info);
-  /* please return it */
-  stream = priv->stream;
-  priv->stream = NULL;
   return stream;
 }
 
